<?php

class actionTopicView extends cmsAction {

    public function run($thread=false){

    	$form = false;

    	$rank_view_allowed = $medal_view_allowed = $rank_add_post = $medal_add_post = true;

    	$user = cmsUser::getInstance();

    	$categories = $this->getForumCategoriesList();
		
    	$thread = $this->model->getForumThreadBySlug($thread);
    	if (!$thread) { cmsCore::error404(); }

    	$category = $this->model->getForumCategoryBySlug($categories[$thread['category_id']]['slug']);
    	if (!$category) { cmsCore::error404(); }

        $thread['thread_sticker'] = $this->getSticker($category['stickers'], $thread['sticker']);

    	//Модератор?
    	$is_moderator = $this->isModerator($user->id, $category['moderators']);

    	if ($is_moderator) { $user->is_moderator = true; }
    	else { $user->is_moderator = false; }

    	//Получаем список рангов
    	$user_ranks = $this->model->resetFilters()->orderBy('ordering')->getAllRanks();

    	//Получаем список медалей
    	$user_medals = $this->model->resetFilters()->orderBy('ordering')->getAllMedals();

    	if ($user->id) {
    		
    		//Проверяем доступность просмотра темы по рангу
    		$rank = $this->showUserRanks($user_ranks, $thread);
    		if (!empty($category['rank_view'])) {
	    		$rank_view_allowed = $this->rankAllowed($rank, $category['rank_view'], $user);
	    	}
	    	if (!empty($category['rank_add_post'])) {
	    		$rank_add_post = $this->rankAllowed($rank, $category['rank_add_post'], $user);
	    	}
    		
    		//Проверяем доступность просмотра темы по медалям
    		$medals = $this->showUserMedals($user_medals, $thread);
    		if (!empty($category['medal_view'])) {
	    		$medal_view_allowed = $this->medalAllowed($medals, $category['medal_view'], $user);
	    	}
	    	if (!empty($category['medal_add_post'])) {
	    		$medal_add_post = $this->medalAllowed($medals, $category['medal_add_post'], $user);
	    	}

    	}
    	
    	//Доступ к теме
    	if ( !isset($categories[$thread['category_id']]) || (!$user->is_admin && !$user->isInGroups(cmsModel::yamlToArray($categories[$thread['category_id']]['groups_view']))) || !$rank_view_allowed || !$medal_view_allowed ) {
    		cmsUser::addSessionMessage(LANG_TOPIC_PRIVATE_PAGE, 'errors');
    		$this->redirectToAction('');
    	}

    	//Подписан на тему?
    	$is_subs = $this->model->isUserSubsByThreadId($user->id, $thread['id']);

    	//Получаем поля
    	$fields = $this->model_content->setTablePrefix('')->orderBy('ordering')->getContentFields('topic_threads');
    	$this->model->resetFilters();

    	$page = $this->request->get('page', 1);
    	$perpage = $this->options['post_perpage'];

    	$this->model->limitPage($page, $perpage);
    	$this->model->orderBy('id');
    	$this->model->filterEqual('thread_id', $thread['id']);

    	$posts = $this->model->getForumThreads($thread['category_id'], 'post');

    	$this->model->filterEqual('thread_id', $thread['id']);
    	$total = $this->model->getForumThreadsCount($thread['category_id'], 'post');

    	if($user->id){

    		//Save Track Info
    		$this->model->saveTrackInfo($thread, $user);

    		if ( ( $user->isInGroups(cmsModel::yamlToArray($category['groups_add_post'])) && $rank_add_post && $medal_add_post ) || $user->is_admin || $is_moderator ) {
    			if (empty($thread['is_closed'])) {

    				if($flood = $this->checkFlood($user, 'post')){
						$thread['no_form_reason'] = LANG_TOPIC_MAX_PUBLIC_RULE.' '.html_spellcount($flood['limit'], $this->options['post_names']).' '.LANG_TOPIC_ZA.' '.html_spellcount($flood['time'], LANG_TOPIC_MINUTE);
    				} else {
    					$form = $this->getForm('message');
    					$form = $this->handlerForm($form, 'post');
    				}

    			} else {
    				$thread['no_form_reason'] = LANG_TOPIC_THREAD_IS_CLOSED;
    			}
    		} else {
    			$thread['no_form_reason'] = LANG_TOPIC_RULE_DNT_NOT_PUBLIC_THREAD;
    		}

    	} else {
    		$thread['no_form_reason'] = LANG_TOPIC_PLEASE_REGISTER_BEFORE_MESSAGE.'. '.LANG_TOPIC_LOGIN_OR_REGISTER.'.';
    	}

    	$item['title'] = $thread['title'];

    	$is_submitted = $this->request->has('submit');
			
		if ($form && $is_submitted){

			$item = $form->parse($this->request, $is_submitted);
				
			$errors = $form->validate($this, $item);

			if (!$errors){

				$item['user_id'] = $user->id;
				$item['thread_id'] = $thread['id'];
				$item['category_id'] = $thread['category_id'];
				$item['date_update'] = date('Y-m-d H:i:s');

				$item = cmsEventsManager::hook('topic_post_before_add', $item);

				//Записываем сообщение в БД
				$item['id'] = $this->model->addForumThread($item);

				//Обновляем кол-во сообщений в категории
				$this->model->updateForumPostCount($thread['category_id']);

				//Обновляем id последнего сообщения в категории
				$this->model->updateLastMessageInCategory($thread['category_id'], $item['id']);

				//Обновляем id последнего сообщения в теме
				$this->model->updateLastMessageInThread($thread['id']);

				//Обновляем кол-во сообщений в теме
				$this->model->updateThreadPostCount($thread['id']);

				//Обновление кол-ва сообщений пользователя
				$this->model->updateUserPostCounter($user->id);

				//Лента активности
				cmsCore::getController('activity')->addEntry($this->name, "add.post", array(
					'subject_title' => $item['title'],
					'subject_url' => href_to_rel('topic', 'thread', $item['id']),
					'subject_id' => $item['id'],
					'user_id' => $user->id
				));

				cmsEventsManager::hook('topic_post_after_add', $item);
				
				cmsUser::addSessionMessage(LANG_TOPIC_MESSAGE_INSERT, 'success');

				//Уведомление о новом сообщении
				$this->sendSubscriptions(2, $item);

				$this->redirectToAction('thread', $item['id']);
					
			} else {
					
				cmsUser::addSessionMessage(LANG_FORM_ERRORS, 'error');
					
			}
				
		}

		//Пользователи он-лайн
    	$this->model->updateUserOnline( array( 'user_id' => $user->id, 'category_id' => $thread['category_id'], 'thread_id' => $thread['id'] ) );

    	$online = $this->model->getTopicUsersOnline( array( 'thread_id' => $thread['id'] ) );

    	//Счетчик хитов
    	$this->model->updateThreadHitsCounter($thread);

    	//Получаем список групп пользователей
    	$user_groups = cmsCore::getModel('users')->getGroups();

    	//Получаем список рангов
    	$user_ranks = $this->model->resetFilters()->orderBy('ordering')->getAllRanks();

    	//Получаем список медалей
    	$user_medals = $this->model->resetFilters()->orderBy('ordering')->getAllMedals();

    	define('CURRENT_TOPIC_CATEGORY_ID', $thread['category_id']);
    	
    	$this->cms_template->render('view', array(
			'thread' => $thread,
			'category' => $category,
			'categories' => $categories,
			'form' => $form,
			'item' => $item,
			'errors' => isset($errors) ? $errors : false,
			'posts' => $posts,
			'page' => $page,
			'perpage' => $perpage,
			'total' => isset($total) ? $total : false,
			'online' => $online,
			'is_moderator' => $is_moderator,
			'is_subs' => $is_subs,
			'fields' => $fields,
			'user_groups' => $user_groups,
			'user_ranks' => $user_ranks,
			'user_medals' => $user_medals
		));

    }

}
