<?php

class modelTopic extends cmsModel {
	
	public function getForumCategories(){

		return $this->get('topic_categories');

	}

	public function getCategoriesCount(){

		return $this->getCount('topic_categories');

	}

	public function filterPermsCategory(){

		$user = cmsUser::getInstance();
		if(!$user->is_admin) {
			array_push($user->groups, 0);
			$this->joinInner('topic_perms_rules', 'p', 'p.category_id = i.id');
			$this->filterIn('p.user_group_id', $user->groups);
		}

	}

	public function filterPermsThread(){

		$user = cmsUser::getInstance();
		if(!$user->is_admin) {
			array_push($user->groups, 0);
			$this->joinInner('topic_perms_rules', 'p', 'p.category_id = i.category_id');
			$this->filterIn('p.user_group_id', $user->groups);
		}

	}

	public function addForumCategory($item){

		return $this->insert('topic_categories', $item);

	}

	public function deleteForumCategory($id){

		$category = self::getForumCategory($id);

		if (!empty($category['icon'])) {

			$icon = self::yamlToArray($category['icon']);
			
			foreach($icon as $path){
				files_delete_file($path, 2);
			}
			
		}

		self::resetParentIdCategories($id, $category['parent_id']);

		self::deleteForumThreadsByCategory($id);

		self::deletePermsGroupView($id);

		return $this->delete('topic_categories', $id);

	}

	public function getForumCategory($id){

		$this->resetFilters();

		return $this->getItemById('topic_categories', $id);

	}

	public function updateForumCategory($id, $item){

		$this->resetFilters();

		return $this->update('topic_categories', $id, $item);

	}

	public function getForumCategoryBySlug($slug){

		$item = $this->getItemByField('topic_categories', 'slug', $slug);

		if (empty($item['moderators'])) { return $item; }

		$item['moderators'] = $this->yamlToArray($item['moderators']);

		$users_model = cmsCore::getModel('users');

		foreach ($item['moderators'] as $key => $value) {
			$mod = $users_model->getUser($value);
			if ($mod) $item['moderators'][$key] = $mod;
		}

		return $item;

	}

	public function getForumThreads($category_id, $type='thread'){

		if($type=='thread'){
			$this->filterIsNull('thread_id');
		} else {
			$this->filterNotNull('thread_id');
		}

		$this->filterEqual('category_id', $category_id);

		$this->joinUser('user_id', array('topic_thread_counter'=>'user_topic_thread_counter', 'topic_post_counter'=>'user_topic_post_counter', 'nickname'=>'user_nickname', 'avatar'=>'user_avatar', 'u.rating'=>'user_rating', 'date_reg'=>'user_date_reg', 'date_log'=>'user_date_log','fsign'=>'user_signature', 'groups' => 'user_groups', 'karma' => 'user_karma', 'friends_count'=> 'user_friends_count', 'is_locked'=>'user_is_locked'));

		return $this->get('topic_threads');

	}

	public function addForumThread($item){

		return $this->insert('topic_threads', $item);

	}

	public function updateForumThread($id, $item){

		return $this->update('topic_threads', $id, $item);

	}

	public function getForumThreadsCount($category_id, $type='thread', $to=false){

		if ($type=='thread'){
			$this->filterIsNull('thread_id');
		} else {
			$this->filterNotNull('thread_id');
		}

		if ($to) {
			$this->filterLtEqual('id', $to);
		}

		$this->filterEqual('category_id', $category_id);

		return $this->getCount('topic_threads');

	}

	public function getForumThreadBySlug($slug){

		$this->joinUser('user_id', array('topic_thread_counter'=>'user_topic_thread_counter', 'topic_post_counter'=>'user_topic_post_counter', 'nickname'=>'user_nickname', 'avatar'=>'user_avatar', 'u.rating'=>'user_rating', 'date_reg'=>'user_date_reg', 'date_log'=>'user_date_log','fsign'=>'user_signature', 'groups' => 'user_groups', 'karma' => 'user_karma', 'friends_count'=> 'user_friends_count', 'is_locked'=>'user_is_locked'));

		$this->join('topic_categories', 'c', 'c.id = i.category_id');
		$this->select('c.title', 'category_title');
		$this->select('c.slug', 'category_slug');

		return $this->getItemByField('topic_threads', 'slug', $slug);

	}

	public function getForumThreadById($id){

		$this->joinUser();

		$this->join('topic_categories', 'c', 'c.id = i.category_id');
		$this->select('c.slug', 'category_slug');

		return $this->getItemById('topic_threads', $id);

	}

	public function deleteForumThread($id){

		self::deleteForumPostsByThread($id);

		$thread = self::getForumThreadById($id);

		$this->resetFilters();

		$this->delete('topic_threads', $id);

		//Лента акивности
		cmsCore::getController('activity')->deleteEntry('topic', "new.thread", $id);

		//Рейтинг
		cmsCore::getModel('rating')->deleteVotes('topic', 'thread', $id);

		//Обновление кол-ва тем пользователя
		self::updateUserThreadCounter($thread['user_id']);

		self::deleteFieldsThread($thread);

	}


	public function deleteForumPostsByThread($id){

		$thread = self::getForumThreadById($id);

		$this->filterEqual('thread_id', $id);

		$posts = self::getForumThreads($thread['category_id'], 'post');

		if ($posts) {

			foreach ($posts as $key => $value) {

				$this->resetFilters();
		
				$this->delete('topic_threads', $key);

				//Обновление кол-ва сообщений пользователя
				self::updateUserPostCounter($value['user_id']);

				//Лента активности
				cmsCore::getController('activity')->deleteEntry('topic', "add.post", $key);

				//Рейтинг
				cmsCore::getModel('rating')->deleteVotes('topic', 'thread', $key);

				


			}

		}

	}

	public function updateForumTopicCount($category_id){

		$topic_count = self::getForumThreadsCount($category_id);

		$this->resetFilters();

		return self::updateForumCategory($category_id, array('topic_counter' => $topic_count));

	}

	public function updateForumPostCount($category_id){

		$post_count = self::getCategoryPostCount($category_id, 'post');

		$this->resetFilters();

		return self::updateForumCategory($category_id, array('post_counter' => $post_count));

	}

	public function resetParentIdCategories($id, $parent_id){

		$this->resetFilters();
		$this->filterEqual('parent_id', $id);

		return $this->updateFiltered('topic_categories', array('parent_id' => $parent_id));

	}

	public function deleteForumThreadsByCategory($id){

		$this->resetFilters();

		return $this->delete('topic_threads', $id, 'category_id');

	}

	public function updateLastMessageInCategory($category_id, $thread){

		if (!$thread) {

			$last = self::getLastThreadInCategory($category_id);
			$thread = $last ? $last['id'] : NULL;

		}

		$this->resetFilters();

		return $this->update('topic_categories', $category_id, array('last_message' => $thread));		

	}

	public function updateLastMessageInThread($thread_id=false){

		$last = self::getLastMessageInThread($thread_id);

		$this->resetFilters();

		return $this->update('topic_threads', $thread_id, array('last_message' => $last['id'], 'date_update' => date('Y-m-d H:i:s')));		

	}

	public function getLastThreadInCategory($category_id){

		$this->resetFilters();
		$this->filterEqual('category_id', $category_id);
		$this->orderBy('id', 'desc');
		$this->limit(1);

		return $this->getItem('topic_threads');

	}

	public function getLastMessageInThread($thread_id){

		$this->resetFilters();
		$this->filterEqual('thread_id', $thread_id);
		$this->orderBy('id', 'desc');
		$this->limit(1);

		return $this->getItem('topic_threads');

	}

	//Получить кол-во сообщений в теме
	public function getThreadPostCount($thread_id){

		$this->resetFilters();

		$this->filterNotNull('thread_id');
		$this->filterEqual('thread_id', $thread_id);

		return $this->getCount('topic_threads');

	}

	//Обновить кол-во сообений в теме
	public function updateThreadPostCount($thread_id){

		$post_count = self::getThreadPostCount($thread_id);

		$this->resetFilters();

		return self::updateForumThread($thread_id, array('post_counter' => $post_count));

	}

	public function getCategoryPostCount($category_id){

		$this->resetFilters();

		$this->filterNotNull('thread_id');
		$this->filterEqual('category_id', $category_id);

		return $this->getCount('topic_threads');

	}

	public function getRatingTarget($subject, $id){

        $item = $this->getForumThreadById($id);

        return $item;

    }

    public function updateRating($subject, $id, $rating){

    	return self::updateForumThread($id, array('rating' => $rating));

    }

    public function updateUserThreadCounter($user_id){

    	$count = self::getUserThreadCounter($user_id);

    	$this->resetFilters();

    	return $this->update('{users}', $user_id, array('topic_thread_counter' => $count));

    }

    public function getUserThreadCounter($user_id){

    	$this->resetFilters();

    	$this->filterEqual('user_id', $user_id);
    	$this->filterIsNull('thread_id');

    	return $this->getCount('topic_threads');

    }

    public function updateUserPostCounter($user_id){

    	$count = self::getUserPostCounter($user_id);

    	$this->resetFilters();

    	return $this->update('{users}', $user_id, array('topic_post_counter' => $count));

    }

    public function getUserPostCounter($user_id){

    	$this->resetFilters();

    	$this->filterEqual('user_id', $user_id);
    	$this->filterNotNull('thread_id');

    	return $this->getCount('topic_threads');

    }

    public function getAllThreadsCount(){

    	$this->filterIsNull('thread_id');

		return $this->getCount('topic_threads');

	}

	public function getAllPostCount(){

		$this->filterNotNull('thread_id');

		return $this->getCount('topic_threads');

	}

	public function updateUserOnline($item){

		if (!$item['user_id']) { return false; }

		$this->resetFilters();

		$insert_data = array(
            'user_id' => $item['user_id']
        );

        $update_data = array(
        	'category_id' => $item['category_id'],
        	'thread_id' => isset($item['thread_id']) ? $item['thread_id'] : 0,
            'date_pub' => null
        );

        $this->insertOrUpdate('topic_online', $insert_data, $update_data);

	}

	public function getTopicUsersOnline($item=false){

		$this->resetFilters();

		$this->filterDateYounger('date_pub', 5, 'minute');

		if (isset($item['category_id']))
			$this->filterEqual('category_id', $item['category_id']);

		if (isset($item['thread_id']))
			$this->filterEqual('thread_id', $item['thread_id']);

		$this->joinUser();

		return $this->get('topic_online');

	}

	public function getAllThreads(){

		$this->join('topic_categories', 'tc', 'tc.id = i.category_id');
		$this->select('tc.title', 'category_title');
		$this->select('tc.slug', 'category_slug');

		$this->joinUser();

		$this->filterIsNull('thread_id');

		return $this->get('topic_threads');

	}

	public function getAllPosts(){

		$this->join('topic_categories', 'tc', 'tc.id = i.category_id');
		$this->select('tc.title', 'category_title');
		$this->select('tc.slug', 'category_slug');

		$this->joinUser();

		$this->filterNotNull('thread_id');

		return $this->get('topic_threads');

	}

	public function isUserSubsByThreadId($user_id, $id){

		$this->resetFilters();

		$this->filterEqual('user_id', $user_id);
		$this->filterEqual('thread_id', $id);

		return $this->getCount('topic_subscribers');

	}

	public function isUserSubsByCategoryId($user_id, $id){

		$this->resetFilters();

		$this->filterEqual('user_id', $user_id);
		$this->filterEqual('category_id', $id);

		return $this->getCount('topic_subscribers');

	}

	public function deleteUserSubsByCategoryId($user_id, $id){

		$this->resetFilters();

		$this->filterEqual('user_id', $user_id);

		return $this->delete('topic_subscribers', $id, 'category_id');

	}

	public function deleteUserSubsByThreadId($user_id, $id){

		$this->resetFilters();

		$this->filterEqual('user_id', $user_id);

		return $this->delete('topic_subscribers', $id, 'thread_id');

	}

	public function insertUserSubsByCategoryId($user_id, $id){

		return $this->insert('topic_subscribers', array('user_id'=>$user_id, 'category_id'=>$id));

	}

	public function insertUserSubsByThreadId($user_id, $id){

		return $this->insert('topic_subscribers', array('user_id'=>$user_id, 'thread_id'=>$id));

	}

	public function deleteSubsByCategoryId($id){

		$this->resetFilters();

		return $this->delete('topic_subscribers', $id, 'category_id');

	}

	public function deleteSubsByThreadId($id){

		$this->resetFilters();

		return $this->delete('topic_subscribers', $id, 'thread_id');

	}

	public function getSubscriptionsCategory($category_id){

		$this->filterIsNull('thread_id');
		$this->filterEqual('category_id', $category_id);

		return $this->get('topic_subscribers');

	}

	public function getSubscriptionsThread($thread_id){

		$this->filterEqual('thread_id', $thread_id);

		return $this->get('topic_subscribers');

	}

	public function getUserThreadSubs($user_id){

		$this->resetFilters();

		$this->filterIsNull('thread_id');
		$this->filterEqual('user_id', $user_id);

		$this->join('topic_categories', 'c', 'c.id = i.category_id');
		$this->select('c.title', 'category_title');
		$this->select('c.slug', 'category_slug');

		return $this->get('topic_subscribers');

	}

	public function getUserPostSubs($user_id){

		$this->resetFilters();

		$this->filterNotNull('thread_id');
		$this->filterEqual('user_id', $user_id);

		$this->join('topic_threads', 't', 't.id = i.thread_id');
		$this->select('t.title', 'thread_title');
		$this->select('t.slug', 'thread_slug');

		return $this->get('topic_subscribers');

	}

	public function getUserAllSubsCount($user_id){

		$this->resetFilters();

		$this->filterEqual('user_id', $user_id);

		return $this->getCount('topic_subscribers');

	}

	public function updatePermsGroupView($id){
		
		self::deletePermsGroupView($id);

		$category = self::getForumCategory($id);

		if (!empty($category['perms'])) {

			$perms = $this->yamlToArray($category['perms']);

			if(!count($perms)) $perms[] = 0;

			foreach($perms as $perm){
				
				$this->insert('topic_perms_rules', array('category_id' => $id, 'user_group_id' => $perm));
				
			}

		}
		
	}

	public function deletePermsGroupView($id) {

		$this->resetFilters();

		return $this->delete('topic_perms_rules', $id, 'category_id');

	}

	public function updateThreadHitsCounter($thread){

		$this->resetFilters();

		$this->update('topic_threads', $thread['id'], array('hits_count'=>$thread['hits_count']+1));

	}

	public function deleteFieldsThread($thread){

		$content_model = cmsCore::getModel('content');
        $content_model->setTablePrefix('');
        $fields = $content_model->getContentFields('topic_threads', $thread['id']);

        if ($fields) {
	        foreach($fields as $field){
	            $field['handler']->delete($thread[$field['name']]);
	        }
    	}

	}

	public function updateAllThreadsCategory($id, $category_id){

		$this->resetFilters();
		$this->update('topic_threads', $id, array('category_id'=>$category_id));
		$this->resetFilters();
		$this->filterEqual('thread_id', $id);
		$this->updateFiltered('topic_threads', array('category_id'=>$category_id));

	}

	public function getAllRanks(){

		return $this->get('topic_rank');

	}
	
	public function addRank($item){

		$this->insert('topic_rank', $item);

	}

	public function deleteRank($id){

		return $this->delete('topic_rank', $id);

	}

	public function getRank($id){

		return $this->getItemById('topic_rank', $id);

	}

	public function updateRank($id, $item){

		$this->update('topic_rank', $id, $item);

	}

	public function saveTrackInfo($thread, $user){

		$this->resetFilters();

		//Удаляем старые записи
		$this->filterEqual('user_id', $user->id)->
					filterEqual('category_id', $thread['category_id'])->
					filterEqual('thread_id', $thread['id'])->
					deleteFiltered('topic_track');

		//Добавляем новую запись
		$this->insert('topic_track', array('user_id'=>$user->id, 'category_id'=>$thread['category_id'], 'thread_id'=>$thread['id']));

	}

	public function getTrackList($user_id, $category_id=false){

		$this->resetFilters();

		$this->joinExcludingLeft('topic_track', 'tt', 'tt.category_id', 'i.category_id', 'tt.thread_id = i.id AND tt.date_pub >= i.date_update AND tt.user_id = '.$user_id);

		$this->filterDateYounger('date_pub', 1, 'month');
		$this->orderBy('id', 'desc');
		$this->filterIsNull('thread_id');

		if ($category_id) {
			$this->filterEqual('category_id', $category_id);
		}

		$list = $this->get('topic_threads');

		if (!$list) { return false; }

		$result['category'] = array_collection_to_list($list, 'category_id');
		$result['thread'] = array_collection_to_list($list, 'id');

		return $result;

	}

	public function setReadCategoryTrack($id, $user_id, $track_list){

		foreach ($track_list as $key => $value) {
			$this->insert('topic_track', array('user_id'=>$user_id, 'category_id'=>$id, 'thread_id'=>$value));
		}

	}

	public function getAllMedals(){

		return $this->get('topic_medal');

	}

	public function addMedal($item){

		return $this->insert('topic_medal', $item);

	}

	public function deleteMedal($id){

		$medal = self::getMedal($id);

		$icon = self::yamlToArray($medal['icon']);
			
		foreach($icon as $path){
			files_delete_file($path, 2);
		}

		return $this->delete('topic_medal', $id);

	}

	public function getMedal($id){

		return $this->getItemById('topic_medal', $id);

	}

	public function updateMedal($id, $item){

		$this->update('topic_medal', $id, $item);

	}

}
